@@ -50,7 +50,43 @@ def print_associate(associate, tran, out):
         out.write("\tNA")
 
 
-def print_coverage(trans, out, out_gff, wigs_f, wigs_r, table_best):
+def compare_ta_genes(tran, genes, out):
+    ass_genes = []
+    if len(genes) != 0:
+        for gene in genes:
+            if (gene.seq_id == tran.seq_id) and (
+                    gene.strand == tran.strand):
+                if ((tran.start <= gene.start) and (
+                        tran.end >= gene.end)) or (
+                        (tran.start >= gene.start) and (
+                        tran.end <= gene.end)) or (
+                        (tran.start <= gene.start) and (
+                        tran.end <= gene.end) and (
+                        tran.end >= gene.start)) or (
+                        (tran.start >= gene.start) and (
+                        tran.start <= gene.end) and (
+                        tran.end >= gene.end)):
+                    if "gene" in gene.attributes.keys():
+                        ass_genes.append(gene.attributes["gene"])
+                    elif "locus_tag" in gene.attributes.keys():
+                        ass_genes.append(gene.attributes["locus_tag"])
+                    else:
+                        ass_genes.append("".join([
+                            "gene:", str(gene.start), "-",
+                            str(gene.end), "_", gene.strand]))
+    if len(ass_genes) != 0:
+        out.write("\t" + ",".join(ass_genes))
+    else:
+        out.write("\tNA")
+
+
+def print_coverage(trans, out, out_gff, wigs_f, wigs_r, gff_file):
+    genes = []
+    if gff_file is not None:
+        gff_f = open(gff_file, "r")
+        for entry in Gff3Parser().entries(gff_f):
+            if (entry.feature == "gene"):
+                genes.append(entry)
     for tran in trans:
         infos = {}
         tran.attributes["detect_lib"] = tran.attributes["detect_lib"].replace(
@@ -58,7 +94,7 @@ def print_coverage(trans, out, out_gff, wigs_f, wigs_r, table_best):
         out.write("\t".join([tran.seq_id, tran.attributes["Name"],
                              str(tran.start), str(tran.end), tran.strand,
                              tran.attributes["detect_lib"]]))
-        print_associate("associated_gene", tran, out)
+        compare_ta_genes(tran, genes, out)
         print_associate("associated_tss", tran, out)
         print_associate("associated_term", tran, out)
         if tran.strand == "+":
@@ -70,18 +106,14 @@ def print_coverage(trans, out, out_gff, wigs_f, wigs_r, table_best):
         best_track = ""
         best_cover = {}
         for track, cover in infos.items():
-            if not table_best:
-                if best != -1:
-                    out.write(";")
-                out.write("{0}(avg={1})".format(
-                          track, str(cover["avg"])))
+            if best != -1:
+                out.write(";")
+            out.write("{0}({1})".format(
+                      track, str(cover["avg"])))
             if cover["avg"] > best:
                 best = cover["avg"]
                 best_track = track
                 best_cover = cover
-        if table_best:
-            out.write("{0}(avg={1})".format(
-                      best_track, str(best_cover["avg"])))
         out.write("\n")
         new_attrs = {}
         for key, value in tran.attributes.items():
@@ -112,18 +144,22 @@ def gen_table_transcript(gff_folder, args_tran):
                        gff.replace(".gff", ".csv")), "w")
             out_gff = open(os.path.join(args_tran.out_folder, "tmp_gff"), "w")
             out_gff.write("##gff-version 3\n")
-            out.write("\t".join(["strain", "Name", "start", "end", "strand",
-                                 "detect_lib_type", "associated_gene",
-                                 "associated_tss", "associated_term",
-                                 "coverage_details"]) + "\n")
+            out.write("\t".join(["Genome", "Name", "Start", "End", "Strand",
+                                 "Detect_lib_type", "Associated_gene",
+                                 "Associated_tss", "Associated_term",
+                                 "Coverage_details"]) + "\n")
             gff_parser = Gff3Parser()
             for entry in gff_parser.entries(th):
                 trans.append(entry)
-            print_coverage(trans, out, out_gff, wigs_f, wigs_r,
-                           args_tran.table_best)
+            if args_tran.gffs is not None:
+                gff_file = os.path.join(args_tran.gffs,
+                                        gff.replace("_transcript", ""))
+                if not os.path.isfile(gff_file):
+                    gff_file = None
+            else:
+                gff_file = None
+            print_coverage(trans, out, out_gff, wigs_f, wigs_r, gff_file)
             out.close()
             out_gff.close()
             shutil.move(os.path.join(args_tran.out_folder, "tmp_gff"),
                         os.path.join(gff_folder, gff))
-    if os.path.exists(os.path.join(args_tran.out_folder, "merge_wigs")):
-        shutil.rmtree(os.path.join(args_tran.out_folder, "merge_wigs"))
