@@ -3,8 +3,8 @@ import sys
 import copy
 import shutil
 import csv
+import re
 from Bio.Seq import Seq
-from Bio.Alphabet import generic_dna
 from annogesiclib.gff3 import Gff3Parser
 
 
@@ -14,6 +14,24 @@ class Helper(object):
     def __init__(self):
         self.gff3parser = Gff3Parser()
 
+    def feature_without_notgene(self, entry):
+        if (entry.feature != "gene") and (
+            entry.feature != "exon") and (
+            entry.feature != "source") and (
+            entry.feature != "region") and (
+            entry.feature != "repeat_region") and (
+            entry.feature != "transcript") and (
+            entry.feature != "STS") and (
+            entry.feature != "remark"):
+            utr_markers = r'[^\'\ \-35]'
+            for sub_f in entry.feature.lower().split("utr"):
+                match = (re.search(utr_markers, sub_f))
+                if match is None:
+                    pass
+                else:
+                    return True
+        return False
+
     def get_strand_name(self, strand):
         '''change the strand name to f/r'''
         name = ""
@@ -62,6 +80,33 @@ class Helper(object):
             fh.close()
             self._fix_break_line(tar, strains)
 
+    def merge_blast_out(self, ref, tar):
+        tmp_out = tar + "_tmp"
+        out = open(tmp_out, "w")
+        file_num = 0
+        for file_ in (ref, tar):
+            start = False
+            finish = False
+            with open(file_) as fh:
+                for line in fh:
+                    check_line = line.strip()
+                    if (check_line.startswith("Query")):
+                        start = True
+                    if (not start) and (not finish) and (file_num == 0):
+                        out.write(line)
+                    if start and (not check_line.startswith("Database")):
+                        out.write(line)
+                    if (check_line.startswith("Database")) and start:
+                        if file_num == 1:
+                            out.write(line)
+                        else:
+                            start = False
+                            finish = True
+            file_num += 1
+        os.remove(tar)
+        os.remove(ref)
+        shutil.move(tmp_out, tar)
+
     def remove_all_content(self, folder, feature, data_type):
         '''remove all files in one folder'''
         for file_ in os.listdir(folder):
@@ -100,6 +145,25 @@ class Helper(object):
                 shutil.rmtree(os.path.join(folder, "tmp"))
             self.remove_all_content(folder, "_folder", "dir")
 
+    def remove_tmp_dir(self, folder):
+        if folder is not None:
+            if os.path.isdir(folder):
+                shutil.rmtree(folder)
+
+    def remove_tmp_dir_ori(self, ori_folder, folder, types):
+        if folder is not None:
+            if os.path.isdir(folder):
+                for file_ in os.listdir(ori_folder):
+                    for type_ in types:
+                        ori_file = os.path.join(ori_folder, file_)
+                        if (file_.endswith(type_)) and (
+                                os.path.isfile(ori_file)):
+                            shutil.move(ori_file, ori_file + "_old")
+                for file_ in os.listdir(folder):
+                    if os.path.isfile(os.path.join(folder, file_)):
+                        shutil.move(os.path.join(folder, file_), ori_folder)
+                shutil.rmtree(folder)
+
     def remove_wigs(self, wigs):
         '''remove the merge wig folder which is generated by ANNOgesic'''
         if wigs:
@@ -123,7 +187,7 @@ class Helper(object):
                 else:
                     filename = data.split("_STRAIN_")
                     if ("reverse" in data) and ("forward" in data):
-                        print("Error: assign reverse or forward wigs!!!")
+                        print("Error: Assign reverse or forward wigs!!!")
                         sys.exit()
                     elif (prefix == filename[-1][:-1 * len(feature)]):
                         if (for_wig_type == "forward"):
@@ -137,17 +201,17 @@ class Helper(object):
         if detect:
             detect = False
         else:
-            print("Warning: no proper file - " + prefix + feature)
+            print("Warning: No proper file - " + prefix + feature)
             return None
 
     def check_make_folder(self, folder):
         '''make new folder (if the folder exists, 
         it will remove it and create new one)'''
-        path = "/".join(folder.split("/")[:-1])
-        folder = folder.split("/")[-1]
-        if folder in os.listdir(path):
-            shutil.rmtree(os.path.join(path, folder))
-        os.mkdir(os.path.join(path, folder))
+        path = os.path.dirname(folder)
+        gen_folder = os.path.basename(folder)
+        if gen_folder in os.listdir(path):
+            shutil.rmtree(os.path.join(path, gen_folder))
+        os.mkdir(os.path.join(path, gen_folder))
 
     def sort_gff(self, gff_file, out_file):
         gffs = []
@@ -196,15 +260,22 @@ class Helper(object):
 
     def check_uni_attributes(self, gff_file):
         '''check the attributes of gff filee, ie ID has to be unique'''
-        print("Checking gff file of {0}".format(gff_file))
+        print("Checking gff file of {0}".format(gff_file.split("/")[-1]))
         gffs = []
         fh = open(gff_file)
+        lens = {}
         for entry in self.gff3parser.entries(fh):
             if (entry.feature == "source") or (
-                    entry.feature == "region"):
-                length = entry.end
-            else:
-                length = None
+                    entry.feature == "region") or (
+                    entry.feature == "remark"):
+                if entry.seq_id not in lens.keys():
+                    lens[entry.seq_id] = entry.end
+                else:
+                    if entry.end > lens[entry.seq_id]:
+                        lens[entry.seq_id] = entry.end 
+                    print("Warning: Detect repeated source/region/remark of {0}! "
+                          "The longest end point is used as the length of the genome.".format(
+                           entry.seq_id))
             gffs.append(entry)
         gffs = sorted(gffs, key=lambda x: (x.seq_id, x.start, x.end, x.strand))
         first = True
@@ -213,15 +284,20 @@ class Helper(object):
         pre_gff = None
         for gff in gffs:
             if (gff.feature != "source") and (
-                    gff.feature != "region"):
-                if length is not None:
-                    if gff.end > length:
+                    gff.feature != "region") and (
+                    gff.feature != "remark"):
+                if gff.seq_id in lens.keys():
+                    if (gff.end > lens[gff.seq_id]):
                         name = "".join([gff.feature, ":", str(gff.start), "-",
                                         str(gff.end), "_", gff.strand])
-                        print("Error: the end point of " + name +
+                        print("Warning: The end point of " + name +
                               " is longer than the length of whole genome.")
-                        print("Please check the gff file.")
-                        sys.exit()
+                if "ID" not in gff.attributes.keys():
+                    gene_name = (gff.feature + ":" + str(gff.start) + 
+                            "-" + str(gff.end) + "_" + gff.strand)
+                    if (gff.feature != "exon") or (gff.feature != "Exon"):
+                        print("Warning: {0} contains no ID information, "
+                              "it may cause error!".format(gene_name))
             if first:
                 first = False
                 self._add_element(ids, "ID", gff)
@@ -230,14 +306,14 @@ class Helper(object):
                 if gff.seq_id == pre_gff.seq_id:
                     if "ID" in gff.attributes.keys():
                         if gff.attributes["ID"] in ids:
-                            print("Warninng: repeat ID {0} "
+                            print("Warning: Repeat ID {0} "
                                   "in gff file!!!".format(
                                       gff.attributes["ID"]))
                         else:
                             self._add_element(ids, "ID", gff)
                     if "locus_tag" in gff.attributes.keys():
                         if gff.attributes["locus_tag"] in ids:
-                            print("Warning:repeat locus_tag {0} "
+                            print("Warning: Repeat locus_tag {0} "
                                   "in gff file!!!".format(
                                       gff.attributes["locus_tag"]))
                         else:
@@ -246,23 +322,38 @@ class Helper(object):
         fh.close()
 
     def _read_fasta(self, fasta_file):
+        seq_ids = []
+        seqs = []
         seq = ""
         with open(fasta_file, "r") as seq_f:
             for line in seq_f:
                 line = line.strip()
                 if line.startswith(">"):
-                    continue
+                    if seq != "":
+                        seqs.append(seq)
+                    seq_ids.append(line[1:])
+                    seq = ""
                 else:
                     seq = seq + line
-        return seq
+        seqs.append(seq)
+        return seq_ids, seqs
 
     def get_seq(self, gff_file, fasta_file, out_file):
         gff_f = open(gff_file, "r")
         out = open(out_file, "w")
-        seq = self._read_fasta(fasta_file)
+        seq_ids, seqs = self._read_fasta(fasta_file)
         num = 0
+        pre_id = ""
         for entry in self.gff3parser.entries(gff_f):
-            gene = self.extract_gene(seq, entry.start, entry.end, entry.strand)
+            if entry.seq_id != pre_id:
+                pre_id = entry.seq_id
+                id_num = 0
+                for seq_id in seq_ids:
+                    if seq_id == entry.seq_id:
+                        break
+                    else:
+                        id_num += 1
+            gene = self.extract_gene(seqs[id_num], entry.start, entry.end, entry.strand)
             if "ID" in entry.attributes.keys():
                 id_ = entry.attributes["ID"]
             else:
@@ -306,6 +397,6 @@ class Helper(object):
                 if seq.startswith(">"):
                     out.write(seq + "\n")
                 else:
-                    dna = Seq(seq, generic_dna)
+                    dna = Seq(seq)
                     out.write(str(dna.translate()) + "\n")
         out.close()
